/*
 *  Copyright (c) 2003 Forward Computing and Control Pty. Ltd.
 *  NSW. Australia,  www.forward.com.au
 *  All rights reserved.
 *  Redistribution of the source of this package, with or without
 *  modification, is NOT permitted.
 *  Redistribution of the compiled java .class files of this package, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions in compiled java .class form must reproduce the above
 *  copyright notice, this list of conditions and the following disclaimer
 *  in the documentation and/or other materials provided with the distribution.
 *  2. If the software is modified, the compiled java .class distribution must
 *  include, in the documentation and/or other materials provided with the
 *  distribution, a statement that the software has been modified and
 *  identify the person who was responsible for the modifications.
 *  3. Neither the name of the author nor Forward Computing and Control may be
 *  used to endorse or promote products derived from this software
 *  without specific prior written permission.
 *  4.  All advertising materials mentioning features or use of this
 *  software must display the following acknowledgment:
 *  "This product includes software developed by Matthew Ford
 *  and Forward Computing and Control Pty. Ltd.
 *  (http://www.forward.com.au)."
 *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 *  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package au.com.forward.runWST;
import org.antlr.stringtemplate.StringTemplateErrorListener;
import java.util.logging.*;

/**
 *  This is the error listener used for RunWST. <br>
 *  It remembers the first error it sees.
 *
 *@author     matthew ford
 *@created    November 27, 2003
 */
public class ErrorLogListener implements StringTemplateErrorListener {

	/**
	 *  The Java logger for this class
	 */
	public final static Logger logger = Logger.getLogger("au.com.forward.runWST.test.ErrorLogListener");

	private String firstErrorMessage = null;
	private String firstWarningMessage = null;
	private Throwable firstException = null;


	/**
	 *  An Error handler. <br>
	 *  Saves the first error it sees. Logs all errors at Level.FINER
	 *
	 *@param  msg  the error message
	 *@param  t    the exception
	 */
	public void error(String msg, Throwable t) {
		if (firstErrorMessage == null) {
			firstErrorMessage = msg;
			firstException = t;  // see if we can catch the stacktrace.
 		}
		logger.log(Level.FINER, "Error: " + msg + "\n" + ((t != null) ? " Exception Message:" + t.getMessage() : ""), t);
		//throw new RunWSTException("Error"+t.getMessage(),t);
	}


	/**
	 *  An Warning handler. <br>
	 *  Just logs the warning at Level.FINER
	 *
	 *@param  msg  the warning message
	 */
	public void warning(String msg) {
		logger.log(Level.FINER, "Warning: " + msg);
	}


	/**
	 *  A debugging handler, superceeded by Java logging.
	 *
	 *@param  msg  the debug message
	 */
	public void debug(String msg) {
		logger.log(Level.FINER, "Debug: " + msg);
	}


	/**
	 *  Gets the first error message seen.
	 *
	 *@return    The firstErrorMessage
	 */
	public String getFirstErrorMessage() {
		return firstErrorMessage;
	}


	/**
	 *  Gets the first exception seen.
	 *
	 *@return    The firstException value
	 */
	public Throwable getFirstException() {
		return firstException;
	}


	/**
	 *  Clears the first error message and exception.
	 */
	public void clearMessages() {
		firstErrorMessage = null;
		firstException = null;
	}

}


