/*
Copyright (c) 2003 Forward Computing and Control Pty. Ltd.
NSW. Australia,  www.forward.com.au
All rights reserved.

Redistribution of the source of this package, with or without
modification, is NOT permitted.

Redistribution of the compiled java .class files of this package, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions in compiled java .class form must reproduce the above
   copyright notice, this list of conditions and the following disclaimer
   in the documentation and/or other materials provided with the distribution.

2. If the software is modified, the compiled java .class distribution must
   include, in the documentation and/or other materials provided with the 
   distribution, a statement that the software has been modified and
   identify the person who was responsible for the modifications.  

3. Neither the name of the author nor Forward Computing and Control may be 
   used to endorse or promote products derived from this software
   without specific prior written permission.

4.  All advertising materials mentioning features or use of this
    software must display the following acknowledgment: 
    "This product includes software developed by Matthew Ford
     and Forward Computing and Control Pty. Ltd. 
    (http://www.forward.com.au)."


THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package au.com.forward.runWST.maps;
import java.util.*;
import java.io.*;
import java.util.regex.*;
import java.util.logging.*;
import org.antlr.stringtemplate.language.ASTExpr;
import java.nio.CharBuffer;

import java.lang.reflect.*;

/**
 *  Support routines for printing attribute maps
 *
 *@author     matthew ford
 *@created    November 26, 2003
 */
public class AttributeToString {

	/**
	 *  Used to get class loader to load css
	 */
	public final static Logger logger = Logger.getLogger("au.com.forward.runWST.maps.AttributeToString");

	private static String indentIncrement = "   ";
	
	public static final int DEFAULT_MAX_LEVEL = 3;

	
	// used to indent printing

	/**
	 *  Sets the indentIncrement attribute of the AttributeToString object
	 *
	 *@param  increment  The new indentIncrement value
	 */
	public void setIndentIncrement(String increment) {
		indentIncrement = increment;
	}

		/**
	 *  Escapes < > and $ for html/xml use<br>
	 *  handles null input.
	 *
	 *@param  input  the string to process
	 *@return        the escaped string, or null if input was null
	 */
	public static String htmlEscape(String input) {
		if (input == null) {
			return null;
		}
		StringBuffer buf = new StringBuffer(input);
		for (int i = 0; i < buf.length(); i++) {
			char c = buf.charAt(i);
			// Escape '&' and '<' and '>' for well-formed HTML output.
			if (c == '<') {
				buf.setCharAt(i, '&');
				buf.insert(i + 1, "lt;");
				i += 3;
				// the extra charaters
			} else if (c == '&') {
				buf.setCharAt(i, '&');
				buf.insert(i + 1, "amp;");
				i += 4;
				// the extra charaters
			} else if (c == '>') {
				buf.setCharAt(i, '&');
				buf.insert(i + 1, "gt;");
				i += 3;
				// the extra charaters
			}
		}
		return buf.toString();
	}




	/**
	 *  Description of the Method
	 *
	 *@param  table  Description of the Parameter
	 *@return        Description of the Return Value
	 */
	public static String attributeHashtableToString(Map table) {
		if (table != null) {
			return hashtableToString(table, "").toString();
		}
		// else {
		return "";
	}


	/**
	 *  Description of the Method
	 *
	 *@param  obj    Description of the Parameter
	 *@return        Description of the Return Value
	 */
	public static String attributeObjectToString(Object obj) {
		return processObject(obj, "").toString();
	}


	/**
	 *  Description of the Method
	 *
	 *@param  list   Description of the Parameter
	 *@return        Description of the Return Value
	 */
	public static String attributeArrayListToString(ArrayList list) {
		if (list != null) {
			return arrayListToString(list, "").toString();
		}
		// else {
		return "";
	}


	/**
	 *  Description of the Method
	 *
	 *@param  map     Description of the Parameter
	 *@param  indent  Description of the Parameter
	 *@return         Description of the Return Value
	 */
	private static StringBuffer hashtableToString(Map map, String indent) {
		StringBuffer rtn = new StringBuffer();
		for (Iterator it = map.keySet().iterator(); it.hasNext(); ) {
			String name = (String) it.next();
			rtn.append(indent + name + " = ");
			Object obj = map.get(name);
			if ((obj instanceof Map) || (obj instanceof ArrayList)) {
				rtn.append("\n");
			}
			rtn.append(processObject(obj, indent + indentIncrement));
			if (!(rtn.charAt(rtn.length() - 1) == '\n')) {
				rtn.append("\n");
			}
		}
		return rtn;
	}


	/**
	 *  Description of the Method
	 *
	 *@param  obj     Description of the Parameter
	 *@param  indent  Description of the Parameter
	 *@return         Description of the Return Value
	 */
	private static StringBuffer processObject(Object obj, String indent) {
		if (obj instanceof ArrayList) {
			return arrayListToString((ArrayList) obj, indent);
		} else if (obj instanceof Map) {
			return hashtableToString((Map) obj, indent);
		} else if (obj == null) {
			return new StringBuffer();
		} else {
			return new StringBuffer("" + obj);
		}
	}


	/**
	 *  Description of the Method
	 *
	 *@param  list    Description of the Parameter
	 *@param  indent  Description of the Parameter
	 *@return         Description of the Return Value
	 */
	private static StringBuffer arrayListToString(ArrayList list, String indent) {
		StringBuffer rtn = new StringBuffer();
		Object[] array = list.toArray();
		for (int i = 0; i < array.length; i++) {
			Object obj = array[i];
			rtn.append(indent + (i + 1) + ". ");
			StringBuffer objStr = processObject(obj, indent + indentIncrement);
			if (obj instanceof Map) {
				// strip leading space
				objStr = new StringBuffer(objStr.toString().trim());
			}
			rtn.append(objStr);
			rtn.append("\n");
		}
		return rtn;
	}


	
	
}


