/*
Copyright (c) 2003 Forward Computing and Control Pty. Ltd.
NSW. Australia,  www.forward.com.au
All rights reserved.

Redistribution of the source of this package, with or without
modification, is NOT permitted.

Redistribution of the compiled java .class files of this package, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions in compiled java .class form must reproduce the above
   copyright notice, this list of conditions and the following disclaimer
   in the documentation and/or other materials provided with the distribution.

2. If the software is modified, the compiled java .class distribution must
   include, in the documentation and/or other materials provided with the 
   distribution, a statement that the software has been modified and
   identify the person who was responsible for the modifications.  

3. Neither the name of the author nor Forward Computing and Control may be 
   used to endorse or promote products derived from this software
   without specific prior written permission.

4.  All advertising materials mentioning features or use of this
    software must display the following acknowledgment: 
    "This product includes software developed by Matthew Ford
     and Forward Computing and Control Pty. Ltd. 
    (http://www.forward.com.au)."


THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package au.com.forward.runWST.misc;

import au.com.forward.runWST.maps.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import org.antlr.stringtemplate.*;

import java.util.*;
import java.lang.reflect.Constructor;

/**
 *  A model that pulls data from a string template hierarchy. This code is
 *  extremely ugly!
 *
 *@author     matthew ford
 *@created    November 27, 2003
 */
public class JTreeAttributeModel implements TreeModel {
	
	IAttributeTreeNode root = null;

	/**
	 *  Constructor for the JTreeAttributeModel object
	 *
	 *@param  st  Description of the Parameter
	 */
	public JTreeAttributeModel(AttributeHashtable st, String label) {
		//String label = "Attributes Loaded...";
		if (st == null) {
			//label = "Error loading attibutes";
		}
		
		root = new RootAttributeHashtable(st,label);
	}




	/**
	 *  Adds a feature to the TreeModelListener attribute of the
	 *  JTreeAttributeModel object
	 *
	 *@param  l  The feature to be added to the TreeModelListener attribute
	 */
	public void addTreeModelListener(TreeModelListener l) {
	}


	/**
	 *
	 *@param  parent  Description of the Parameter
	 *@param  index   Description of the Parameter
	 *@return         The child value
	 */
	public Object getChild(Object parent, int index) {
		//System.out.println("Get index "+index+" of "+parent.toString()+":"+parent.getClass().getName());
		if (parent == null) {
			return null;
		}
		return AttributeTreeNode.getAttributeTreeNode(parent).getChild(parent, index);
	}


	/**
	 *  Gets the childCount attribute of the JTreeAttributeModel object
	 *
	 *@param  parent  Description of the Parameter
	 *@return         The childCount value
	 */
	public int getChildCount(Object parent) {
		if (parent == null) {
			throw new IllegalArgumentException("root is null");
		}
		return AttributeTreeNode.getAttributeTreeNode(parent).getChildCount(parent);
	}


	/**
	 *  Gets the indexOfChild attribute of the JTreeAttributeModel object
	 *
	 *@param  parent  Description of the Parameter
	 *@param  child   Description of the Parameter
	 *@return         The indexOfChild value
	 */
	public int getIndexOfChild(Object parent, Object child) {
		if (parent == null || child == null) {
			throw new IllegalArgumentException("root or child is null");
		}
		return AttributeTreeNode.getAttributeTreeNode(parent).getIndexOfChild(parent, child);
	}
  

	/**
	 *  Gets the root attribute of the JTreeAttributeModel object
	 *
	 *@return    The root value
	 */
	public Object getRoot() {
		return root;
	}


	/**
	 *  Gets the leaf attribute of the JTreeAttributeModel object
	 *
	 *@param  node  Description of the Parameter
	 *@return       The leaf value
	 */
	public boolean isLeaf(Object node) {
		if (node == null) {
			throw new IllegalArgumentException("node is null");
		}
		return AttributeTreeNode.getAttributeTreeNode(node).isLeaf(node);
	}


	/**
	 *  Description of the Method
	 *
	 *@param  l  Description of the Parameter
	 */
	public void removeTreeModelListener(TreeModelListener l) {
	}


	/**
	 *  Description of the Method
	 *
	 *@param  path      Description of the Parameter
	 *@param  newValue  Description of the Parameter
	 */
	public void valueForPathChanged(TreePath path, Object newValue) {
		throw new IllegalStateException("Not implemented");
		// not used
	}
	
	static class RootAttributeHashtable implements IAttributeTreeNode {
		AttributeHashtable table = null;
		String label;
		
		public RootAttributeHashtable(Object o, String label) {
			this.table = (AttributeHashtable)o;
			if (label == null) {
				throw new IllegalArgumentException("Tree label cannot be null.");
			}
			label = label.trim();
			if (label.length()==0) {
				throw new IllegalArgumentException("Tree label cannot be empty.");
			}
			this.label = label;
		}
		
		public Object getChild(Object parent, int index) {
			String[] attributes = getTableAsStringArrayOfKeys();
			String key = attributes[index];
			return new AttributeMapEntry(key, table.get(key));
		}
		public int getChildCount(Object parent) {
			String[] attributes = getTableAsStringArrayOfKeys();
			return attributes.length;
		}
		public int getIndexOfChild(Object parent, Object child) {
			String[] attributes = getTableAsStringArrayOfKeys();
			if (!(child instanceof String)) {
				return -1;
			}
			String childString = (String)child;
			for (int i=0; i<attributes.length; i++ ) {
				if (childString.equals(attributes[i])) {
					return i;
				}
			}
			return -1;
		}
		
		public boolean isLeaf(Object node) {
			return false;
		}
		public String toString() {
			return label;
		}
		
		private String[] getTableAsStringArrayOfKeys() {
			return table.getTableAsStringArrayOfKeys();
		}
		

	}
	
	
}

