/*
 *  Copyright (c) 2003 Forward Computing and Control Pty. Ltd.
 *  NSW. Australia,  www.forward.com.au
 *  All rights reserved.
 *  Redistribution of the source of this package, with or without
 *  modification, is NOT permitted.
 *  Redistribution of the compiled java .class files of this package, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions in compiled java .class form must reproduce the above
 *  copyright notice, this list of conditions and the following disclaimer
 *  in the documentation and/or other materials provided with the distribution.
 *  2. If the software is modified, the compiled java .class distribution must
 *  include, in the documentation and/or other materials provided with the
 *  distribution, a statement that the software has been modified and
 *  identify the person who was responsible for the modifications.
 *  3. Neither the name of the author nor Forward Computing and Control may be
 *  used to endorse or promote products derived from this software
 *  without specific prior written permission.
 *  4.  All advertising materials mentioning features or use of this
 *  software must display the following acknowledgment:
 *  "This product includes software developed by Matthew Ford
 *  and Forward Computing and Control Pty. Ltd.
 *  (http://www.forward.com.au)."
 *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 *  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package au.com.forward.runWST.wstAttributeReader;

import java.util.*;
import java.util.logging.*;
import org.antlr.stringtemplate.*;
import org.xml.sax.*;
import au.com.forward.runWST.maps.*;
import antlr.*;
import au.com.forward.webstringtemplate.filters.WSTPlainFilter;
import au.com.forward.webstringtemplate.WebStringTemplate;

/**
 *  A map state for HTML extend from this overwrite replace processData()
 *
 *@author     matthew ford
 *@created    November 26, 2003
 */
public class HtmlMapState implements java.io.Serializable {

	/**
	 *  Description of the Field
	 */
	public final static Logger logger = Logger.getLogger("au.com.forward.runWST.wstAttributeReader.HtmlMapState");

	private Stack stack = new Stack();

	private StringTemplateErrorListener errorListener = null;

	private Object value = null;
	// used for adding list and map objects

	private StackItem currentStackItem = new StackItem(null, null, State.NO_MAP_FOUND);

	private boolean extraMaps = false;

	private boolean foundError = false;


	/**
	 *  Process the data before saving in the map. <br>
	 *  This version just removes nulls.
	 *
	 *@param  data  Description of the Parameter
	 *@return       Description of the Return Value
	 */
	public String processData(String data) {
		if (data == null) {
			data = "";
		}
		//return StringEscapeUtils.unescapeHtml(data.trim());
		return WebStringTemplate.loadConvert(data).trim();
	}



	/**
	 *  Gets the errorListener attribute of the HtmlMapState object
	 *
	 *@return    The errorListener value
	 */
	public StringTemplateErrorListener getErrorListener() {
		return errorListener;
	}


	/**
	 *  Gets the table attribute of the HtmlMapState object
	 *
	 *@return    The table value
	 */
	public AttributeHashtable getTable() {
		logger.entering(this.getClass().getName(), "getTable");
		logger.log(Level.FINER, "currentStackItem", currentStackItem);
		if (currentState().equals(State.FINISHED)) {
			logger.exiting(this.getClass().getName(), "getTable", currentObject());
			return (AttributeHashtable) currentObject();
		}
		// else return
		logger.exiting(this.getClass().getName(), "getTable", " null return");
		return null;
	}


	/**
	 *  Gets the current Hashtable or ArrayList
	 *
	 *@return    Get the current Hashtable or ArrayList
	 */
	private Object currentEntry() {
		return currentStackItem.getHashtableOrArrayList();
	}


	/**
	 *  Get the current State
	 *
	 *@return    The current State
	 */
	private State currentState() {
		return currentStackItem.getState();
	}


	/**
	 *  Sets the currentState
	 *
	 *@param  state  The new currentState
	 */
	private void setCurrentState(State state) {
		currentStackItem.setState(state);
	}


	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	private Object currentObject() {
		return currentStackItem.getObject();
	}


	/**
	 *  Sets the currentObject attribute of the HtmlMapState object
	 *
	 *@param  obj  The new currentObject value
	 */
	private void setCurrentObject(Object obj) {
		currentStackItem.setObject(obj);
	}


	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Return Value
	 */
	private String currentName() {
		return currentStackItem.getName();
	}


	/**
	 *  Sets the name attribute of the HtmlMapState object
	 *
	 *@param  name  The new name value
	 */
	private void setName(String name) {
		currentStackItem.setName(name);
	}


	/**
	 *  Constructor for the MapState object
	 *
	 *@param  listener  Description of the Parameter
	 */
	public HtmlMapState(StringTemplateErrorListener listener) {
		errorListener = listener;
	}


	private String currentData = "";

	// hold the accumulate pcdata

	/**
	 *  DL push current state and start new hashtable /DL save map and pop state,
	 *  clear data DT set state, clear name, clear data /DT save name, clear name,
	 *  clear data DD set state, clear data /DD save map item, clear name, clear
	 *  data OL push current state and start new arraylist, set state, clear data
	 *  /OL save list and pop state, clear data UL push current state and start new
	 *  arraylist, set state, clear data /UL save list and pop state, clear data LI
	 *  set state, clear data /LI save list item, clear data termination of current
	 *  states called before starting each new state IN_DL save map and pop state ,
	 *  clear data IN_DT save name and pop state, clear data IN_DD save map item,
	 *  clear name, clear data IN_OL save list and pop state, clear data IN_UL save
	 *  list and pop state, clear data IN_LI add list item, clear data
	 *
	 *@param  tok  Description of the Parameter
	 */
	public void startDL(Token tok) {
		logger.entering(this.getClass().getName(), "startDL", tok);
		if (currentState().equals(State.FINISHED)) {
			if (!extraMaps && !foundError) {
				warning("Ignoring all maps after the first one.", tok);
				extraMaps = true;
				// don't print this again.
			}
		} else if (foundError) {
			return;
		} else if (currentState().equals(State.NO_MAP_FOUND)) {
			logger.finer("Start map");
			setCurrentState(State.FINISHED);
			AttributeHashtable map = new AttributeHashtable();
			setCurrentObject(map);
			push(map, null, State.IN_DL);
		} else if (currentState().gtEqInDl()) {
			// terminate current item and start new map
			// if in DT throw error
			if ((!currentState().equals(State.IN_DD)) &&
					(!currentState().equals(State.IN_LI))) {
				error("Error: Found <DL> outside <DD> or <LI>", tok);
				return;
			} else {
				// dl in dd or li
				logger.finer("push and start new map");
				State lastState = currentState();
				String data = currentData;
				push(new AttributeHashtable(), null, State.IN_DL);
				if (lastState.equals(State.IN_DD) ||
				    lastState.equals(State.IN_LI) ) {
					logger.finer("add toString of '"+data+"'");
					// in dd and starting dl save the current pcdata as map toString()
					String toStringData = processData(data);
					if (toStringData.length() > 0) {
						((Map) currentObject()).put("", new WSTPlainFilter(toStringData));
						if (logger.isLoggable(Level.FINER)) {
							logger.finer(" put('','" + processData(data) + "')");
						}
					}	
				}
					
			}
		} else {
			error("Error: Invalid state:" + currentState(), tok);
		}

	}


	/**
	 *  Description of the Method
	 *
	 *@param  tok  Description of the Parameter
	 */
	public void endDL(Token tok) {
		logger.entering(this.getClass().getName(), "endDL", tok);
		if (currentState().equals(State.FINISHED)) {
			return;
		} else if (foundError) {
			return;
		}
		if (currentState().equals(State.IN_DD)) {
			logger.finer(" terminate DD");
			endDD(tok);
		}
		currentData = "";
		if (currentState().equals(State.IN_DL)) {
			pop();
		} else {
			error("Error: Missing <DL>", tok);
		}
		logger.log(Level.FINER, "currentStackItem", currentStackItem);
	}


	/**
	 *  Description of the Method
	 *
	 *@param  data  Description of the Parameter
	 */
	public void pcdata(String data) {
		logger.entering(this.getClass().getName(), "pcdata", data);
		if (currentState().equals(State.FINISHED)) {
			return;
		} else if (foundError) {
			return;
		}
		if (currentState().gtEqInDl()) {
			currentData += data;
			logger.finer("in pcdata with data:'" + data + "'");
		} else {
			logger.finer("in pcdata, not in map.");
		}
	}


	/**
	 *  Description of the Method
	 *
	 *@param  tok  Description of the Parameter
	 */
	public void startDT(Token tok) {
		logger.entering(this.getClass().getName(), "startDT", tok);
		if (currentState().equals(State.FINISHED)) {
			return;
		} else if (foundError) {
			return;
		}
		if (currentState().equals(State.IN_DD)) {
			logger.finer(" terminate DD");
			endDD(tok);
		}
		if (!(currentState().equals(State.IN_DL))) {
			error("Error: Missing <DL>", tok);
		}
		setName("");
		setCurrentState(State.IN_DT);
		logger.log(Level.FINER, "currentStackItem", currentStackItem);
	}


	/**
	 *  Description of the Method
	 *
	 *@param  tok  Description of the Parameter
	 */
	private void endDT(Token tok) {
		logger.entering(this.getClass().getName(), "endDT", tok);
		if (currentState().equals(State.FINISHED)) {
			return;
		} else if (currentState().equals(State.NO_MAP_FOUND)) {
			return;
		} else if (foundError) {
			return;
		}
		currentData = currentData.trim();
		if (currentData.length() <= 0) {
			error("Attribute name cannot be empty or blank.", tok);
		}
		// only these chars are allowed by the stringtemplate parser
		if (!WSTAttributeReader.attributeNamePattern.matcher(currentData).matches()) {
			error("Error: Attribute name must be letter or _ followed by letter _ or digit", tok);
		}
		setName(currentData);
		currentData = "";
		setCurrentState(State.END_DT);
		logger.log(Level.FINER, "currentStackItem", currentStackItem);
	}


	/**
	 *  Description of the Method
	 *
	 *@param  tok  Description of the Parameter
	 */
	public void startDD(Token tok) {
		logger.entering(this.getClass().getName(), "startDD", tok);
		if (currentState().equals(State.FINISHED)) {
			return;
		} else if (currentState().equals(State.NO_MAP_FOUND)) {
			error("Error: Missing <DL>", tok);
			return;
		} else if (foundError) {
			return;
		}
		if (currentState().equals(State.IN_DT)) {
			logger.finer(" terminate DT");
			endDT(tok);
		}
		//
		if (!currentState().equals(State.END_DT)) {
			error("Error: Must specify any Attribute name for this item. (missing <DT>)", tok);
		}
		setCurrentState(State.IN_DD);
		logger.log(Level.FINER, "currentStackItem", currentStackItem);
	}


	/**
	 *  Description of the Method
	 *
	 *@param  tok  Description of the Parameter
	 */
	private void endDD(Token tok) {
		logger.entering(this.getClass().getName(), "endDD", tok);
		if (currentState().equals(State.FINISHED)) {
			return;
		} else if (currentState().equals(State.NO_MAP_FOUND)) {
			return;
		} else if (foundError) {
			return;
		}
		if (!(currentState().equals(State.IN_DD))) {
			error("Error: Missing <DD>", tok);
			return;
		}
		if (!(currentObject() instanceof Map)) {
			error("Error: Missing <DL>", tok);
			return;
		}
		// else
		if (value != null) {
			((Map) currentObject()).put(WebStringTemplate.loadConvert(currentName()), new WSTPlainFilter(value));
			logger.finer(" put('" + currentName() + "'," + value.getClass().getName() + ")");
		} else {
			// use current data
			((Map) currentObject()).put(WebStringTemplate.loadConvert(currentName()), new WSTPlainFilter(processData(currentData)));
			if (logger.isLoggable(Level.FINER)) {
				logger.finer(" put('" + currentName() + "','" + processData(currentData) + "')");
			}
		}
		value = null;
		currentData = "";
		setName("");
		setCurrentState(State.IN_DL);
		logger.log(Level.FINER, "currentStackItem", currentStackItem);
	}


	/**
	 *  Description of the Method
	 *
	 *@param  tok  Description of the Parameter
	 */
	public void startOL(Token tok) {
		logger.entering(this.getClass().getName(), "startOL", tok);
		if (currentState().equals(State.FINISHED)) {
			return;
		} else if (currentState().equals(State.NO_MAP_FOUND)) {
			return;
		} else if (foundError) {
			return;
		}
		push(new AttributeArrayList(), State.IN_OL);
		logger.log(Level.FINER, "currentStackItem", currentStackItem);
	}


	/**
	 *  Description of the Method
	 *
	 *@param  tok  Description of the Parameter
	 */
	public void endOL(Token tok) {
		logger.entering(this.getClass().getName(), "endOL", tok);
		if (currentState().equals(State.FINISHED)) {
			return;
		} else if (currentState().equals(State.NO_MAP_FOUND)) {
			return;
		} else if (foundError) {
			return;
		}
		if (!(currentState().equals(State.END_LI))) {
			logger.finer(" terminate LI");
			endLI(tok);
		}
		if (currentState().equals(State.END_LI)) {
			pop();
		}
		currentData = "";
		logger.log(Level.FINER, "currentStackItem", currentStackItem);
	}


	/**
	 *  Description of the Method
	 *
	 *@param  tok  Description of the Parameter
	 */
	public void startUL(Token tok) {
		logger.entering(this.getClass().getName(), "startUL", tok);
		if (currentState().equals(State.FINISHED)) {
			return;
		} else if (currentState().equals(State.NO_MAP_FOUND)) {
			return;
		} else if (foundError) {
			return;
		}
		push(new AttributeArrayList(), State.IN_UL);
		logger.log(Level.FINER, "currentStackItem", currentStackItem);
	}


	/**
	 *  Description of the Method
	 *
	 *@param  tok  Description of the Parameter
	 */
	public void endUL(Token tok) {
		logger.entering(this.getClass().getName(), "endUL", tok);
		if (currentState().equals(State.FINISHED)) {
			return;
		} else if (currentState().equals(State.NO_MAP_FOUND)) {
			return;
		} else if (foundError) {
			return;
		}
		if (!(currentState().equals(State.END_LI))) {
			logger.finer(" terminate LI");
			endLI(tok);
		}
		if (currentState().equals(State.END_LI)) {
			pop();
		}
		currentData = "";
		logger.log(Level.FINER, "currentStackItem", currentStackItem);
	}


	/**
	 *  Description of the Method
	 *
	 *@param  tok  Description of the Parameter
	 */
	public void startLI(Token tok) {
		logger.entering(this.getClass().getName(), "startLI", tok);
		if (currentState().equals(State.FINISHED)) {
			return;
		} else if (currentState().equals(State.NO_MAP_FOUND)) {
			return;
		} else if (foundError) {
			return;
		}
		if ((!(currentState().equals(State.IN_OL))) &&
				(!(currentState().equals(State.IN_UL))) &&
				(!(currentState().equals(State.END_LI)))
				) {
			logger.finer(" terminate LI");
			endLI(tok);
		}
		currentData = "";
		setCurrentState(State.IN_LI);
		logger.log(Level.FINER, "currentStackItem", currentStackItem);
	}


	/**
	 *  Description of the Method
	 *
	 *@param  tok  Description of the Parameter
	 */
	private void endLI(Token tok) {
		logger.entering(this.getClass().getName(), "endLI", tok);
		if (currentState().equals(State.FINISHED)) {
			return;
		} else if (currentState().equals(State.NO_MAP_FOUND)) {
			return;
		} else if (foundError) {
			return;
		}
		if (!(currentState().equals(State.IN_LI))) {
			error("Error: Missing <LI>", tok);
			return;
		}
		if (value != null) {
			((AttributeArrayList) currentObject()).add(value);
			logger.finer(" add(" + value.getClass().getName() + ")");
		} else {
			((AttributeArrayList) currentObject()).add(processData(currentData));
			logger.finer(" add(" + processData(currentData) + ")");
		}
		value = null;
		currentData = "";
		setCurrentState(State.END_LI);
		logger.log(Level.FINER, "currentStackItem", currentStackItem);
	}



	/**
	 *  Description of the Method
	 *
	 *@param  tok  Description of the Parameter
	 */
	public void endBody(Token tok) {
		logger.entering(this.getClass().getName(), "endBODY", tok);
		while ((currentStackItem != null) &&
				(!currentState().equals(State.FINISHED)) &&
				(!(currentState().equals(State.NO_MAP_FOUND))) &&
				(!foundError)
				) {
			terminateCurrentState(tok);
		}
		logger.log(Level.FINER, "currentStackItem", currentStackItem);
	}


	/**
	 *  Description of the Method
	 *
	 *@param  tok  Description of the Parameter
	 */
	private void terminateCurrentState(Token tok) {
		State state = currentState();
		if (state.equals(State.IN_UL)) {
			endUL(tok);
		} else if (state.equals(State.IN_OL)) {
			endOL(tok);
		} else if (state.equals(State.IN_DD)) {
			endDD(tok);
		} else if (state.equals(State.IN_DT)) {
			endDT(tok);
		} else if (state.equals(State.IN_DL)) {
			endDL(tok);
		} else if (state.equals(State.IN_LI)) {
			endLI(tok);
		} else if (state.equals(State.END_DT)) {
			endLI(tok);
		} else if (state.equals(State.END_LI)) {
			endLI(tok);
		}
	}


	// don't need start body because only look for tags inside body tag


	/**
	 *  Description of the Method
	 *
	 *@param  hash         Description of the Parameter
	 *@param  currentName  Description of the Parameter
	 *@param  state        Description of the Parameter
	 */
	private void push(Map hash, String currentName, State state) {
		stack.push(currentStackItem);
		logger.log(Level.FINER, "pushed currentStackItem", currentStackItem);
		currentStackItem = new StackItem(hash, currentName, state);
		setName("");
		currentData = "";
	}


	/**
	 *  Description of the Method
	 *
	 *@param  list   Description of the Parameter
	 *@param  state  Description of the Parameter
	 */
	private void push(ArrayList list, State state) {
		stack.push(currentStackItem);
		logger.log(Level.FINER, "pushed currentStackItem", currentStackItem);
		currentStackItem = new StackItem(list, state);
		setName("");
		currentData = "";
	}


	/**
	 *  Description of the Method
	 */
	public void pop() {
		// save current entry
		value = currentObject();
		if (!(stack.isEmpty())) {
			currentStackItem = (StackItem) stack.pop();
		} else {
			currentStackItem = null;
		}
		logger.log(Level.FINER, "popped currentStackItem", currentStackItem);
	}



	// U T I L I T Y  R O U T I N E S

	/**
	 *  Description of the Method
	 *
	 *@param  msg  Description of the Parameter
	 *@param  tok  Description of the Parameter
	 */
	public void error(String msg, Token tok) {
		error(msg, null, tok);
	}


	/**
	 *  Description of the Method
	 *
	 *@param  msg  Description of the Parameter
	 *@param  tok  Description of the Parameter
	 */
	public void warning(String msg, Token tok) {
		if (errorListener != null) {
			errorListener.warning(getLocation(tok) + msg);
		} else {
			System.err.println("HtmlMapState: warning: " + getLocation(tok) + msg);
		}
	}


	/**
	 *  Description of the Method
	 *
	 *@param  msg  Description of the Parameter
	 *@param  tok  Description of the Parameter
	 */
	public void debug(String msg, Token tok) {
		if (errorListener != null) {
			errorListener.debug(getLocation(tok) + msg);
		} else {
			System.err.println("HtmlMapState: debug: " + getLocation(tok) + msg);
		}
	}


	/**
	 *  Description of the Method
	 *
	 *@param  msg  Description of the Parameter
	 *@param  e    Description of the Parameter
	 *@param  tok  Description of the Parameter
	 */
	public void error(String msg, Exception e, Token tok) {
		foundError = true;
		if (errorListener != null) {
			errorListener.error(getLocation(tok) + msg, e);
		} else {
			if (e != null) {
				System.err.println("HtmlMapState: error: " + getLocation(tok) + msg + ": " + e.getMessage());
			} else {
				System.err.println("HtmlMapState: error: " + getLocation(tok) + msg);
			}
		}
	}


	/**
	 *  get the location of the current element
	 *
	 *@param  tok  Description of the Parameter
	 *@return      The location value
	 */
	private String getLocation(Token tok) {
		StringBuffer str = new StringBuffer();
		if (tok == null) {
			return "No location information available";
		} else {
			return "line:" + tok.getLine() + " col:" + tok.getColumn() + " ";
		}

	}


	/**
	 *  Description of the Class
	 *
	 *@author     matthew ford
	 *@created    November 26, 2003
	 */
	protected class StackItem {
		/**
		 * the name of the object if it is a Map, else null
		 */
		String name = null;
		
		/**
		 * the state to be saved and popped later
		 */
		State state = State.NO_MAP_FOUND;
		
		/**
		 * the Map or ArrayList to be save and popped later
		 */
		Object hashtableOrArrayList = null;


		/**
		 *  Constructor for the StackItem object
		 */
		private StackItem() { }


		/**
		 *  toString Method
		 *
		 *@return    Description of the this object
		 */
		public String toString() {
			String objStr = "null";
			if (hashtableOrArrayList instanceof AttributeHashtable) {
				objStr = "AttributeHashtable " + ((AttributeHashtable) hashtableOrArrayList).toString();
			} else if (hashtableOrArrayList instanceof AttributeArrayList) {
				objStr = "AttributeArrayList " + ((AttributeArrayList) hashtableOrArrayList).toString();
			} else if (hashtableOrArrayList != null) {
				objStr = "Unknown object. " + hashtableOrArrayList.getClass().getName();
			}

			return "State:" + state + " name:" + ((name == null) ? "null" : name) + " " + objStr;
		}


		/**
		 *  Constructor for the StackItem object
		 *
		 *@param  hash   the new map
		 *@param  name   name of the map
		 *@param  state  current state
		 */
		public StackItem(Map hash, String name, State state) {
			this.name = name;
			// name for this map item will be null if obj is a list.
			this.hashtableOrArrayList = hash;
			this.state = state;
		}


		/**
		 *  Constructor for the StackItem object
		 *
		 *@param  list   the new ArrayList
		 *@param  state  the current state
		 */
		public StackItem(ArrayList list, State state) {
			this.name = null;
			// name for this map item will be null if obj is a list.
			this.hashtableOrArrayList = list;
			this.state = state;
		}


		/**
		 *  Gets the name of the current object. <br>
		 *  Will be null for ArrayList
		 *
		 *@return    The name value
		 */
		public String getName() {
			return name;
		}


		/**
		 *  Sets the name StackItem object.
		 *
		 *@param  name  The new name value
		 */
		public void setName(String name) {
			if (name == null) {
				name = "";
			}
			this.name = name;
		}


		/**
		 *  Gets the state of the StackItem object
		 *
		 *@return    The state
		 */
		public State getState() {
			return state;
		}


		/**
		 *  Sets the state the StackItem object
		 *
		 *@param  state  The new state value
		 */
		public void setState(State state) {
			this.state = state;
		}


		/**
		 *  Gets the object, Map or ArrayList
		 *
		 *@return    The object value
		 */
		public Object getObject() {
			return hashtableOrArrayList;
		}


		/**
		 *  Sets the object, Map or ArrayList
		 *
		 *@param  object  The new object value
		 */
		public void setObject(Object object) {
			this.hashtableOrArrayList = object;
		}


		/**
		 *  Gets the hashtableOrArrayListthe StackItem object
		 *
		 *@return    The hashtableOrArrayList value
		 */
		public Object getHashtableOrArrayList() {
			return hashtableOrArrayList;
		}
	}





}


