/*
Modified by Matthew Ford for WebStringTemplates
Copyright (c) 2003 Forward Computing and Control Pty. Ltd.
NSW. Australia,  www.forward.com.au
All rights reserved.

Redistribution of the source of this package, with or without
modification, is NOT permitted.

Redistribution of the compiled java .class files of this package, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions in compiled java .class form must reproduce the above
   copyright notice, this list of conditions and the following disclaimer
   in the documentation and/or other materials provided with the distribution.

2. If the software is modified, the compiled java .class distribution must
   include, in the documentation and/or other materials provided with the 
   distribution, a statement that the software has been modified and
   identify the person who was responsible for the modifications.  

3. Neither the name of the author nor Forward Computing and Control may be 
   used to endorse or promote products derived from this software
   without specific prior written permission.

4.  All advertising materials mentioning features or use of this
    software must display the following acknowledgment: 
    "This product includes software developed by Matthew Ford
     and Forward Computing and Control Pty. Ltd. 
    (http://www.forward.com.au)."


THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*	
	Based on the HTML 3.2 spec. by the W3 (http://www.w3.org)
	Alexander Hinds & Terence Parr
	Magelang Institute, Ltd.
	Send comments to:  parrt@jguru.com
	
	All tags are stripped out
	use excapedHtml to insert tags in to data.
	accept either ordered or unorder list as a list but treat at ordered

*/
header {
/*
Copyright (c) 2003 Forward Computing and Control Pty. Ltd.
NSW. Australia,  www.forward.com.au
All rights reserved.

Redistribution of the source of this package, with or without
modification, is NOT permitted.

Redistribution of the compiled java .class files of this package, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions in compiled java .class form must reproduce the above
   copyright notice, this list of conditions and the following disclaimer
   in the documentation and/or other materials provided with the distribution.

2. If the software is modified, the compiled java .class distribution must
   include, in the documentation and/or other materials provided with the 
   distribution, a statement that the software has been modified and
   identify the person who was responsible for the modifications.  

3. Neither the name of the author nor Forward Computing and Control may be 
   used to endorse or promote products derived from this software
   without specific prior written permission.

4.  All advertising materials mentioning features or use of this
    software must display the following acknowledgment: 
    "This product includes software developed by Matthew Ford
     and Forward Computing and Control Pty. Ltd. 
    (http://www.forward.com.au)."


THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package au.com.forward.runWST.wstAttributeReader.language;
import au.com.forward.runWST.wstAttributeReader.*;
import org.antlr.stringtemplate.*;
}
class HTMLParser extends Parser;
options {
	exportVocab=HTML;
	k = 1;
}

{
  HtmlMapState mapState = null;
	
  StringTemplateErrorListener listener = null;
	
	public void setMapState(HtmlMapState mapState) {
   if (mapState == null) {
    throw new IllegalStateException("HtmlMapState cannot be null.");
	 }
		this.mapState = mapState;
		this.listener = mapState.getErrorListener();
	}
	
	
  public void reportError(RecognitionException e) {
    listener.error("line:"+e.getLine()+" col:"+e.getColumn()+" "+e.getMessage(),null);
  }

	
}

document
 { 
   if (mapState == null){
    throw new IllegalStateException("Need to set HtmlMapState for this parser.");
	 }
   if (listener == null){
    throw new IllegalStateException("Need to set StringTemplateErrorListener for this parser.");
	 }
 }
	: 	(pcdata)* 
		//(script | style)*
		(body)?
		{mapState.endBody(null);}
	;


body
   ://( OBODY (pcdata)* )? 
		( body_content )+ 
		//( CBODY (pcdata)* )? 
	;	



body_content
	: 	list (pcdata)*
	;

pcdata
  : pc:PCDATA {mapState.pcdata(pc.getText());}
	;
	
list:	unordered_list
	|	ordered_list
	|	def_list
	;

unordered_list
	:	ou:OULIST {mapState.startUL(ou);} (pcdata)* (list_item)+ cu:CULIST {mapState.endUL(cu);}
	;

ordered_list
	:	oo:OOLIST {mapState.startOL(oo);} (pcdata)* (list_item)+ co:COLIST {mapState.endOL(co);}
	;

def_list
	:	od:ODLIST {mapState.startDL(od);} (pcdata)* (def_list_item)+ cd:CDLIST {mapState.endDL(cd);} 
	;

list_item
	:	ol:OLITEM {mapState.startLI(ol);} ( pcdata | list )* (cl:CLITEM (pcdata)*)?
	;
	
def_list_item
	:	dt dd
	;

dt	:	od:ODTERM {mapState.startDT(od);} (pcdata)+ (cd:CDTERM (pcdata)*)?
	;

dd	:	od:ODDEF {mapState.startDD(od);} (pcdata|list)* (CDDEF (pcdata)*)?
	;

/*TEXT ELEMENTS*/

	

class HTMLLexer extends Lexer;
options {	
	k = 2;
	exportVocab=HTML;
	charVocabulary = '\3'..'\377';
	caseSensitive=false;
	//filter=UNDEFINED_TOKEN;
}

{
  private boolean debug = false;
  private boolean escFlag = false;  // true if in escape block
	private int openLine = 0; // line of last open <wst_esc>
	private int openColumn = 0; // column of last open <wst_esc>
	public boolean isEscape() {
		return escFlag;
	}
	public int getOpenLine() {
		return openLine;
	}
	public int getOpenColumn() {
		return openColumn;
	}
	public void uponEOF() throws TokenStreamException, CharStreamException {
		if (isEscape()) {
			 throw new RuntimeException("Missing </wst> for tag opened at Line:"
			 +getOpenLine()+" Column:"+getOpenColumn());
		}
	}	
	/**
	 * check start of txt against token and then looks for white space in next char
	 * input string must already be lowercase
	 */
	private boolean checkWS(String txt, String token) {
	  if (txt.startsWith(token)) {
		  int len = token.length();
			if (txt.length() > len) {
			  char c = txt.charAt(len);
				if ((debug) &&
				   (Character.isWhitespace(c) || c=='>') ) {
					System.out.println("matchedWS '"+txt+"'");
				}
				return (Character.isWhitespace(c) || c=='>');
			}
		}
		return false;
	}
	
	/**
	 * checks txt against token
	 * input string must already be lowercase
	 */
	private boolean check(String txt, String token) {
		if ((debug) && (txt.equals(token))) {
				System.out.println("matched '"+txt+"'");
		}
	  return (txt.equals(token));
	}
	
}


TAG
 : ('<')=> ( 
		 ("<!--")=>  COMMENT { $setType(PCDATA);}
		 | ("<wst>")=>  WST_COMMENT { $setType(PCDATA);}
		 // | ("<wst/>")=> "<wst/>"! { $setType(PCDATA);}
	   | OTHER_TAG 
	   {
	     // check text and set type
			 // check OULIST, CULIST, OOLIST, COLIST, ODLIST, CDLIST, OLITEM, CLITEM, ODTERM, CDTERM, ODDEF, CDDEF
			String txt = $getText;
			// make it lowercase 
			txt = txt.toLowerCase();
			if (checkWS(txt,"<ul")) {
			   $setType(OULIST);
			} else if (check(txt,"</ul>")) {
			   $setType(CULIST);
			} else if (checkWS(txt,"<ol")) {
			   $setType(OOLIST);
			} else if (check(txt,"</ol>")) {
			   $setType(COLIST);
			} else if (checkWS(txt,"<dl")) {
			   $setType(ODLIST);
			} else if (check(txt,"</dl>")) {
			   $setType(CDLIST);
			} else if (checkWS(txt,"<li")) {
			   $setType(OLITEM);
			} else if (check(txt,"</li>")) {
			   $setType(CLITEM);
			} else if (checkWS(txt,"<dt")) {
			   $setType(ODTERM);
			} else if (check(txt,"</dt>")) {
			   $setType(CDTERM);
			} else if (checkWS(txt,"<dd")) {
			   $setType(ODDEF);
			} else if (check(txt,"</dd>")) {
			   $setType(CDDEF);
			} else {
			   $setType(PCDATA);
			}	 
			//System.out.println(txt); 
	   }
   )	
 | PCDATA { $setType(PCDATA);}
 ;	

protected
OTHER_TAG
	:	(
			/* See comment in WS.  Language for combining any flavor
			 * newline is ambiguous.  Shutting off the warning.
			 */
			options {
				generateAmbigWarnings=false;
			}
		:	'\r' '\n'		{newline();}
		|	'\r'			{newline();}
		|	'\n'			{newline();}
		|	~('>'|'\n'|'\r') 
		)+
	'>' { $setType(PCDATA); }
	;

	
// wst data
protected
WST_COMMENT_DATA
	:	(	/*	'\r' '\n' can be matched in one alternative or by matching
				'\r' in one iteration and '\n' in another.  I am trying to
				handle any flavor of newline that comes in, but the language
				that allows both "\r\n" and "\r" and "\n" to all be valid
				newline is ambiguous.  Consequently, the resulting grammar
				must be ambiguous.  I'm shutting this warning off.
			 */
			options {
				generateAmbigWarnings=false;
			}
		:
			{ (LA(2)!='/') 
			  || (LA(2) == '/' && LA(3)!='w')
				|| (LA(2) == '/' && LA(3)=='w' && LA(4)!='s')
				|| (LA(2) == '/' && LA(3)=='w' && LA(4)=='s'&& LA(5)!='t')
				|| (LA(2) == '/' && LA(3)=='w' && LA(4)=='s' && LA(5)=='t' && LA(6)!='>')}? '<' // allow '<' if not "</wst>"
		|	'\r' '\n'		{newline();}
		|	'\r'			{newline();}
		|	'\n'			{newline();}
		|	~('<'|'\n'|'\r')
		)*
	;


protected
WST_COMMENT
	:	"<wst>"!
	  {
			escFlag = true;
			openLine = getLine();
			openColumn = getColumn()-($getText).length();
		}	
   	WST_COMMENT_DATA
		"</wst>"!
			{ 
				escFlag = false;
				openLine = 0;
				openColumn = 0;
				$setType(PCDATA);
			}
	;
	

protected 
OWST
options {
  paraphrase = "<WST>";
}
	:	"<wst>"	
	{
	      escFlag = true;
				$setType(PCDATA);
				openLine = getLine()+1;
				openColumn = getColumn()-($getText).length();
				setText("");
			} 
	;

protected 
CWST
options {
  paraphrase = "</WST>";
}
	:	"</wst>" {
	    escFlag = false;
			$setType(PCDATA);
			setText("");
			openLine = 0;
			openColumn = 0;
     }
	;


		/*UNORDERED LIST*/
protected		
OULIST
options {
  paraphrase = "<UL>";
}
	:	"<ul" (WS ATTR)? '>' 
	;

protected
CULIST
options {
  paraphrase = "</UL>";
}
	:	"</ul>"
	;

		/*ORDERED LIST*/
protected
OOLIST
options {
  paraphrase = "<OL>";
}
	:	"<ol" (WS ATTR)? '>'
	;

protected
COLIST
options {
  paraphrase = "</OL>";
}
	:	"</ol>"
	;

		/*LIST ITEM*/

protected
OLITEM
options {
  paraphrase = "<LI>";
}
	:	"<li" (WS ATTR)? '>'
	;

protected
CLITEM
options {
  paraphrase = "</LI>";
}
	:	"</li>"
	;

		/*DEFINITION LIST*/ 

protected
ODLIST 
options {
  paraphrase = "<DL>";
}
	:	"<dl" (WS ATTR)? '>' 
	;

protected
CDLIST
options {
  paraphrase = "</DL>";
}
	:	"</dl>"
	;

protected
ODTERM
options {
  paraphrase = "<DT>";
}
	: 	"<dt>"
	;

protected
CDTERM
options {
  paraphrase = "</DT>";
}
	: 	"</dt>"
	;

protected
ODDEF
options {
  paraphrase = "<DD>";
}
	: 	"<dd>"
	;

protected
CDDEF
options {
  paraphrase = "</DD>";
}
	: 	"</dd>"
	;

/*	pcdata-LEVEL ELEMENTS
*/



protected
PCDATA
options {
  paraphrase = "text";
}
	:	(
			/* See comment in WS.  Language for combining any flavor
			 * newline is ambiguous.  Shutting off the warning.
			 */
			options {
				generateAmbigWarnings=false;
			}
		:	'\r' '\n'		{newline();}
		|	'\r'			{newline();}
		|	'\n'			{newline();}
		|	~('<'|'\n'|'\r') // was ~('<'|'\n'|'\r'|'"'|'>') but that missed " and closing >
		)+
	{
	 if (debug) {
	  //System.out.println("PCDATA '"+$getText+"'");
	 }
	}

	;

// multiple-line comments
protected
COMMENT_DATA
	:	(	/*	'\r' '\n' can be matched in one alternative or by matching
				'\r' in one iteration and '\n' in another.  I am trying to
				handle any flavor of newline that comes in, but the language
				that allows both "\r\n" and "\r" and "\n" to all be valid
				newline is ambiguous.  Consequently, the resulting grammar
				must be ambiguous.  I'm shutting this warning off.
			 */
			options {
				generateAmbigWarnings=false;
			}
		:
			{(LA(2)!='-') 
			 || (LA(2)=='-' && LA(3)!='>') }? '-' // allow '-' and "-?>" and "--?" but not "-->"
		//	{LA(2)!='-' && LA(3)!='>'}? '-' // allow '-' if not "-->"
		|	'\r' '\n'		{newline();}
		|	'\r'			{newline();}
		|	'\n'			{newline();}
		|	~('-'|'\n'|'\r')
		)*
	;


protected
COMMENT
	:	"<!--" COMMENT_DATA "-->"	{ $setType(Token.SKIP); }
	;

/*
	PROTECTED LEXER RULES
*/

protected
WS	:	(
			/*	'\r' '\n' can be matched in one alternative or by matching
				'\r' in one iteration and '\n' in another.  I am trying to
				handle any flavor of newline that comes in, but the language
				that allows both "\r\n" and "\r" and "\n" to all be valid
				newline is ambiguous.  Consequently, the resulting grammar
				must be ambiguous.  I'm shutting this warning off.
			 */
			options {
				generateAmbigWarnings=false;
			}
		:	' '
		|	'\t'
		|	'\n'	{ newline(); }
		|	"\r\n"	{ newline(); }
		|	'\r'	{ newline(); }
		)+
	;

protected
ATTR
options {
ignore=WS;
}
	:	WORD ('=' (WORD ('%')? | ('-')? INT | STRING | HEXNUM))?
	;

//don't need uppercase for case-insen.
//the '.' is for words like "image.gif"
protected
WORD:	(	LCLETTER
		|	'.' | '/'
		)

		(
			/*	In reality, a WORD must be followed by whitespace, '=', or
				what can follow an ATTR such as '>'.  In writing this grammar,
				however, we just list all the possibilities as optional
				elements.  This is loose, allowing the case where nothing is
				matched after a WORD and then the (ATTR)* loop means the
				grammar would allow "widthheight" as WORD WORD or WORD, hence,
				an ambiguity.  Naturally, ANTLR will consume the input as soon
				as possible, combing "widthheight" into one WORD.

				I am shutting off the ambiguity here because ANTLR does the
				right thing.  The exit path is ambiguous with ever
				alternative.  The only solution would be to write an unnatural
				grammar (lots of extra productions) that laid out the
				possibilities explicitly, preventing the bogus WORD followed
				immediately by WORD without whitespace etc...
			 */
			options {
				generateAmbigWarnings=false;
			}
		:	LCLETTER
		|	DIGIT
		|	'.'
		)+
	;

protected
STRING
	:	'"' (~'"')* '"'
	|	'\'' (~'\'')* '\''
	;

protected
WSCHARS
	:	' ' | '\t' | '\n' | '\r'
	;

protected 
SPECIAL
	:	'<' | '~'
	;
	
protected
HEXNUM
	:	'#' HEXINT
	;

protected
INT	:	(DIGIT)+
	;

protected
HEXINT
	:	(
			/*	Technically, HEXINT cannot be followed by a..f, but due to our
				loose grammar, the whitespace that normally would follow this
				rule is optional.  ANTLR reports that #4FACE could parse as
				HEXINT "#4" followed by WORD "FACE", which is clearly bogus.
				ANTLR does the right thing by consuming a much input as
				possible here.  I shut the warning off.
			 */
			 options {
				generateAmbigWarnings=false;
			}
		:	HEXDIGIT
		)+
	;

protected
DIGIT
	:	'0'..'9'
	;

protected
HEXDIGIT
	:	'0'..'9'
	|	'a'..'f'
	;

protected
LCLETTER
	:	'a'..'z'
	;	

protected
UNDEFINED_TOKEN
	:	( "\r\n" | '\r' | '\n' ) {newline();}
	|	.
	;


