/*
 *  Copyright (c) 2003 Forward Computing and Control Pty. Ltd.
 *  NSW. Australia,  www.forward.com.au
 *  All rights reserved.
 *  Redistribution of the source of this package, with or without
 *  modification, is NOT permitted.
 *  Redistribution of the compiled java .class files of this package, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions in compiled java .class form must reproduce the above
 *  copyright notice, this list of conditions and the following disclaimer
 *  in the documentation and/or other materials provided with the distribution.
 *  2. If the software is modified, the compiled java .class distribution must
 *  include, in the documentation and/or other materials provided with the
 *  distribution, a statement that the software has been modified and
 *  identify the person who was responsible for the modifications.
 *  3. Neither the name of the author nor Forward Computing and Control may be
 *  used to endorse or promote products derived from this software
 *  without specific prior written permission.
 *  4.  All advertising materials mentioning features or use of this
 *  software must display the following acknowledgment:
 *  "This product includes software developed by Matthew Ford
 *  and Forward Computing and Control Pty. Ltd.
 *  (http://www.forward.com.au)."
 *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 *  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package au.com.forward.webstringtemplate.filters;
import java.util.*;
import java.util.logging.*;
import au.com.forward.logging.*;

/**
 *  This filter marks an attribute as missing but allows for
 *  a path and error return string
 *
 *@author     matthew ford
 *@created    November 26, 2003
 */
public class WSTMissingFilter extends WSTPlainFilter {

	/**
	 *  The Java logger for this class
	 */
	public final static Logger logger = Logger.getLogger("au.com.forward.webstringtemplate.filters.WSTMissingFilter");


	/**
	 *  Default constructor for the WSTMissingFilter object
	 */
	public WSTMissingFilter() { }


	/**
	 *  Constructor for the WSTMissingFilter object
	 *
	 *@param  obj   the object to filter, can be null
	 */
	public WSTMissingFilter(Object obj) {
		super(obj);
	}

	/**
	 *  Constructor for the WSTMissingFilter object
	 *
	 *@param  obj   the object to filter, can be null
	 */
	public WSTMissingFilter(Object[] obj) {
		super(obj);
	}

	/**
	 *  Returns a new instance of this filter. <br>
	 *  The object and path themselves are not copied. This usually correct as the object will be
	 *  reset and the path. Any other other objects in the filter are need to be copied across
	 *  in the over of this method.
	 *
	 *@return    a new instance of this filter
	 */
	public IWSTFilter newInstance() {
		WSTMissingFilter missingFilter = (WSTMissingFilter)super.newInstance();
		// do any other initialization here
		return missingFilter;
	}
	
	/**
	 *  Just return the object.toString().<br>
	 *  Should be the error return path.
	 *
	 *@return    the error return path.	 
	 */
	public String toString() {
		logger.entering(this.getClass().getName(), "toString", obj);
		if (obj instanceof IWSTFilter) {
			// filters do not chain directly
			// to chain filters first convert to string and then wrap the string in the next filter.
			return obj.toString();
		}
		return super.toString();
	}

}

